# coding: UTF-8
#jtalk/translateTest.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2012 Takuya Nishimoto (NVDA Japanese Team)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
# 
# Usage for Test:
# > python source\synthDrivers\jtalk\translateTest.py
#
# 数 283C   3456 ⠼ 数符
# .         2      小数点
# '         3      位取り点、アポストロフィ
#
# 外 2830   56   ⠰ 外字符
# 大 2820   6    ⠠ 大文字符
# ＜ 2826   236  ⠦ 外国語引用符 全角小なり
# ＞ 2834   356  ⠴ 外国語引用符(とじる) 全角大なり
#
# 。        256-0-0 句点
# 、        56-0    読点
# ・        5       中点
# ？        26-0-0  疑問符
# ！        235-0-0 感嘆符
# － 2824   36   ⠤ 第一つなぎ符 全角マイナス
#           6-3     第二つなぎ符
#
# 「 2824   36   ⠤ 第一かぎ
# 」 2824   36   ⠤ 第一かぎ閉じる
# 『        56-36   第二かぎ
# 』        36-23   第二かぎ閉じる
# （        2356    第一カッコ
# ）        2356    第一カッコ閉じる
#           5-2356  第二カッコ
#           2356-2  第二カッコ閉じる
#           56-2356 二重カッコ
#           2356-23 二重カッコ閉じる
#
#           25-25   棒線
#           2-2-2   点線
#           25-25-135  右矢印
#           246-25-25  左矢印
#           246-25-25-135  両向き矢印
# ～        36-36   波線
# ％        56-1234 パーセント(pと同じ)
# ＆        56-12346 アンドマーク
# ＃        56-146   ナンバーマーク
# ＊        56-16    アステリスク
#
#           6-236    情報処理用点字
#           6-356    情報処理用点字閉じる
#           4        行継続符
# :         5-2      情報処理用点字 コロン
# /         34       情報処理用点字 スラッシュ
# @         246      情報処理用点字 アットマーク
# _         5-36     情報処理用点字 アンダーライン
# ~         5-14     情報処理用点字 オーバーライン、チルダ

import sys
import os
from mecab import *

_logwrite = None

try:
	from logHandler import log
	_logwrite = log.info
except:
	def __print(s): print s.encode('utf-8', 'ignore')
	_logwrite = __print

def Mecab_split_v2(feature, size):
	if feature is None or size is None: 
		return
	hyouki_list  = []
	nhyouki_list = []
	hinshi1_list = []
	hinshi2_list = []
	kana_list    = []
	yomi_list    = []
	output_list  = []
	sepflag_list = []
	for i in xrange(0, size):
		s = string_at(feature[i])
		if s:
			s = s.decode(CODE, 'ignore')
			#_logwrite(s)
			ar = s.split(",")
			hyouki_list .append(ar[0])
			nhyouki_list.append(unicodedata.normalize('NFKC',ar[0]))
			hinshi1_list.append(ar[1])
			hinshi2_list.append(ar[2])
			if len(ar) > 9:
				kana_list   .append(ar[8])
				yomi_list   .append(ar[9])
				output_list .append(ar[9])
			else:
				# 100,名詞,数,*,*,*,*,*
				kana_list   .append(ar[0])
				yomi_list   .append(ar[0])
				output_list .append(ar[0])
			sepflag_list.append(False)
	prev_hinshi1 = None
	for i in xrange(0, len(yomi_list)):
		if hinshi1_list[i] == u'空白':
			sepflag_list[i] = True
		if hinshi1_list[i] == u'数':
			if hyouki_list[i] != nhyouki_list[i]:
				# digit numbers (not kanji characters)
				output_list[i] = nhyouki_list[i]
		if prev_hinshi1 == u'形容詞' and hinshi1_list[i] == u'名詞':
			sepflag_list[i-1] = True
		if prev_hinshi1 == u'助詞' and hinshi1_list[i] == u'名詞':
			sepflag_list[i-1] = True
		if prev_hinshi1 == u'助詞' and hinshi1_list[i] == u'動詞':
			sepflag_list[i-1] = True
		if prev_hinshi1 == u'動詞' and hinshi1_list[i] == u'形容詞':
			sepflag_list[i-1] = True
		prev_hinshi1 = hinshi1_list[i]
	# 
	s2 = ''
	for i in xrange(0, len(output_list)):
		#_logwrite("%d %s %s %s" % (i, hinshi1_list[i], sepflag_list[i], nhyouki_list[i]))
		s2 += output_list[i]
		if sepflag_list[i] == True: s2 += ' '
	#_logwrite("s2:%s" % s2)
	return re.sub("  ", " ", s2.strip())

def braille_transcription(text):
	Mecab_initialize(_logwrite)
	try:
		text = unicodedata.normalize('NFKC',text)
		text = text.encode(CODE, 'ignore')
		mf = MecabFeatures()
		Mecab_analysis(text, mf)
		_logwrite('')
		Mecab_print(mf, _logwrite)
		text = Mecab_split_v2(mf.feature, mf.size)
		mf = None
		#_logwrite(text)
	except Exception, e:
		raise # _logwrite('[translate error] ' + str(e))
		text = ''
	return text

def test_braille():
	tests = [
		[u"NVDA 2010.2 ユーザーガイド", u"外大大nvda 数2010. 数2 ユーザー ガイド"],
		[u"一般的な機能", u"イッパンテキナ キノー"],
		[u"国際化", u"コクサイカ"],
		[u"音声合成サポート", u"オンセイ ゴーセイ サポート"],
		[u"点字ディスプレイサポート", u"テンジ ディスプレイ サポート"],
		[u"ライセンスと著作権", u"ライセンスト チョサクケン"],
		[u"システム要件", u"システム ヨーケン"],
		[u"NVDAの入手", u"外大大nvda ノ ニューシュ"],
		[u"NVDA入門", u"外大大nvda ニューモン"],
		[u"NVDAの起動", u"外大大nvda ノ キドー"],
		[u"コマンドについて", u"コマンドニ ツイテ"],
		[u"フォーカスによる移動", u"フォーカスニ ヨル イドー"],
		[u"システムキャレットによる移動", u"システム キャレットニ ヨル イドー"],
		[u"オブジェクト内の移動", u"オブジェクトナイノ イドー"],
		[u"現在オブジェクトのテキストの確認", u"ゲンザイ オブジェクトノ テキストノ カクニン"],
		[u"マウスによる移動", u"マウスニ ヨル イドー"],
		[u"仮想バッファー", u"カソー バッファー"],
		[u"単一文字によるナビゲーション", u"タンイツ モジニ ヨル ナビゲーション"],
		[u"フォームとフォーカスモード", u"フォームト フォーカス モード"],
		[u"要素リスト", u"ヨーソ リスト"],
		[u"埋め込みオブジェクト", u"ウメコミ オブジェクト"],
		[u"設定メニュー", u"セッテイ メニュー"],
		[u"一般設定", u"イッパン セッテイ"],
		[u"出力先の切り替え", u"シュツリョクサキノ キリカエ"],
		[u"音声設定", u"オンセイ セッテイ"],
		[u"点字設定", u"テンジ セッテイ"],
		[u"読み上げ辞書", u"ヨミアゲ ジショ"],
		[u"さらに詳細な情報", u"サラニ ショーサイナ ジョーホー"],
		[u"NVDA 2010.2 User Guideの日本語訳です。", u"＜大大nvda 数2010. 数2 大user 大guide＞ ノ ニホンゴヤクデス。"],
		[u"正式な文書ではないことにご注意下さい。", u"セイシキナ ブンショデワ ナイ コトニ ゴチューイ クダサイ。"],
		[u"また、翻訳において生じた誤りが含まれる可能性があります。", u"マタ、 ホンヤクニ オイテ ショージタ アヤマリガ フクマレル カノーセイガ アリマス。"],
		[u"合成音声によるフィードバックによって、", u"ゴーセイ オンセイニ ヨル フィード バックニ ヨッテ、"],
		[u"全盲者をはじめとした視覚障害者が", u"ゼンモーシャヲ ハジメト シタ シカク ショーガイシャガ"],
		[u"オペレーティングシステムや、さまざまなサードパーティー製のアプリケーションを利用することを可能にします。", u"オペレーティング システムヤ、 サマザマナ サード パーティーセイノ アプリケーションヲ リヨー スル コトヲ カノーニ シマス。"],
		[u"主なハイライトは以下の通りです。", u"オモナ ハイライトワ イカノ トオリデス。"],
		[u"インストールなしでUSBメモリースティックや他の携帯型メディアによる実行が可能", u"インストール ナシデ 外大大usb メモリー スティックヤ ホカノ ケイタイガタ メディアニ ヨル ジッコーガ カノー"],
		[u"簡単に利用できるトーキングインストーラー", u"カンタンニ リヨー デキル トーキング インストーラー"],
		[u"アクセシブルなJavaアプリケーションのサポート", u"アクセシブルナ ＜大java＞ アプリケーションノ サポート"],
		[u"Adobe Readerのサポート", u"＜大adobe 大reader＞ ノ サポート"],
		[u"マウス位置のテキストの自動読み上げおよびマウス位置をビープ音で知らせるオプション", u"マウス イチノ テキストノ ジドー ヨミアゲ オヨビ マウス イチヲ ビープオンデ シラセル オプション"],
		[u"多くの点字ディスプレイのサポート", u"オオクノ テンジ ディスプレイノ サポート"],
		[u"日本語版は未対応", u"ニホンゴバンワ ミタイオー"],
		[u"世界中の誰もが、どんな言語を話しているかに関わらず、等しくテクノロジーにアクセスできるということが重要です。", u"セカイジューノ ダレモガ、 ドンナ ゲンゴヲ ハナシテ イルカニ カカワラズ、 ヒトシク テクノロジーニ アクセス デキルト イウ コトガ ジューヨーデス。"],
		[u"NVDAは現在、以下の言語を含む20ヶ国語以上の言語に翻訳されています。", u"外大大nvda ワ ゲンザイ、 イカノ ゲンゴヲ フクム 数20カコクゴ イジョーノ ゲンゴニ ホンヤク サレテ イマス。"],
		[u"メッセージやインターフェースを複数の言語で提供できるだけでなく、もし利用者が使用したい言語に対応した音声エンジンを持っていれば、NVDAではどんな言語のコンテンツも読ませることができます。", u"メッセージヤ インター フェースヲ フクスーノ ゲンゴデ テイキョー デキルダケデ ナク、 モシ リヨーシャガ シヨー シタイ ゲンゴニ タイオー シタ オンセイ エンジンヲ モッテ イレバ、 外大大nvda デワ ドンナ ゲンゴノ コンテンツモ ヨマセル コトガ デキマス。"],
		[u"NVDAは、無料でオープンソースの多言語対応音声合成ソフトウェアであるeSpeakをバンドルしています。", u"外大大nvda ワ、 ムリョーデ オープン ソースノ タゲンゴ タイオー オンセイ ゴーセイ ソフトウェアデ アル ＜e大speak＞ ヲ バンドル シテ イマス。"],
		[u"加えてNVDAでは音声読み上げを行うためにSAPI4とSAPI5の両方の音声エンジンが利用できる他、Audiologic speech synthesizerやNewFon speech synthesizerも利用できます。", u"クワエテ 外大大nvda デワ オンセイ ヨミアゲヲ オコナウ タメニ 外大大sapi数4ト 外大大sapi数5ノ リョーホーノ オンセイ エンジンガ リヨー デキル ホカ、 ＜大audiologic speech synthesizer＞ヤ ＜大new大fon speech synthesizer＞モ リヨー デキマス。"],
#		[u"", u""],
		[u"スタートメニュー「プログラム」にある", u"スタート メニュー 「プログラム」ニ アル"],
#		[u"", u""],
		[u"広い海を泳ぐ小さい魚。", u"ヒロイ ウミヲ オヨグ チーサイ サカナ。"],
		[u"美しい自然。", u"ウツクシイ シゼン。"],
		[u"百円ショップは何でも１００円。", u"数100－エン ショップワ ナンデモ 数100－エン。"],
		[u"ねえ、先生。", u"ネエ、 センセイ。"],
		[u"母は花屋へ行きました。", u"ハハワ ハナヤエ イキマシタ。"],
		[u"字を書く", u"ジヲ カク"],
		[u"お母さん", u"オカアサン"],
		[u"映画", u"エイガ"],
		[u"空気", u"クーキ"],
		[u"お父さん", u"オトーサン"],
		[u"閏年", u"ウルウドシ"],
		[u"思う", u"オモウ"],
		[u"ありがとう", u"アリガトー"],
		[u"狼", u"オオカミ"],
		[u"通る", u"トオル"],
		[u"多い", u"オオイ"],
		[u"大晦日", u"オオミソカ"],
		[u"手作り", u"テヅクリ"],
		[u"インタビュー", u"インタビュー"],
		[u"日本点字図書館", u"ニッポン テンジ トショカン"],
		[u"一円硬貨", u"数1エン コーカ"],
		[u"10メートル四方", u"数10メートル 数4ホー"],
		[u"十メートル四方", u"数10メートル 数4ホー"],
		[u"百人一首", u"数100ニン 数1シュ"],
		[u"十七歳", u"数17サイ"],
		[u"17歳", u"数17サイ"],
		[u"五つ星", u"イツツボシ"],
		[u"三、四日", u"数3数4カ"],
		[u"ボーイング777", u"ボーイング 数777"],
		[u"徒歩で五分", u"トホデ 数5フン"],
		[u"500kHz", u"数500外k大hz"],
		[u"AbCD", u"外大ab大大cd"],
		[u"二十五度C", u"数25ド外大c"],
		[u"JR線", u"外大大jr－セン"],
		[u"逆V字型", u"ギャク外大v－ジガタ"],
		[u"TOEFLテスト", u"外大大toefl テスト"],
		[u"C-90", u"外大c－数90"],
		[u"新聞や雑誌などなど", u"シンブンヤ ザッシナドナド"],
		[u"今日は晴れるでしょう", u"キョーワ ハレルデショー"],
		[u"50へ手が届く", u"数50－エ テガ トドク"],
		[u"見てる", u"ミテル"],
		[u"見ている", u"ミテ イル"],
		[u"歩き始める", u"アルキハジメル"],
		[u"理論的すぎる", u"リロンテキスギル"],
		[u"利用できる", u"リヨー デキル"],
		[u"運動をする", u"ウンドーヲ スル"],
		[u"意図する", u"イト スル"],
		[u"読書しない", u"ドクショ シナイ"],
		[u"面白おかしい", u"オモシロオカシイ"],
		[u"赤くなる", u"アカク ナル"],
		[u"問題ない", u"モンダイ ナイ"],
		[u"山の手", u"ヤマノテ"],
		[u"受付窓口", u"ウケツケ マドグチ"],
		[u"六法全書", u"数6ポー ゼンショ"],
		[u"アイスクリーム", u"アイス クリーム"],
		[u"仮名文字", u"カナモジ"],
		[u"南半球", u"ミナミ ハンキュー"],
		[u"北半球", u"キタハンキュー"],
		[u"事務機器", u"ジム キキ"],
		[u"右半分", u"ミギ ハンブン"],
		[u"株式会社", u"カブシキガイシャ"],
		[u"小中学校", u"ショーチューガッコー"],
		[u"都道府県", u"トドーフケン"],
		[u"二〇〇〇年一月一日", u"数2000ネン 数1ガツ ツイタチ"],
		[u"ルイ・ブライユ", u"ルイ ブライユ"],
		[u"アメリカ合衆国", u"アメリカ ガッシューコク"],
		[u"小一時間", u"コ数1ジカン"],
		[u"第一人者", u"ダイ数1ニンシャ"],
		[u"全五巻", u"ゼン数5カン"],
		[u"超一流", u"チョー 数1リュー"],
		[u"一人一人", u"ヒトリ ヒトリ"],
		[u"盲", u"モー"],
		[u"聾", u"ロー"],
	]
	count_ok = 0
	count_all = 0
	for i in tests:
		s = braille_transcription(i[0])
		count_all += 1
		correct = 0
		if s == i[1]: 
			count_ok += 1
			correct = 1
		_logwrite("  '%s'" % i[0])
		_logwrite("  '%s'" % i[1])
		_logwrite("%d '%s'" % (correct, s))
	_logwrite('')
	_logwrite("%d / %d passed" % (count_ok, count_all))

if __name__ == "__main__":
	test_braille()
